const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");

const debug = Cc["@mozilla.org/js/jsd/debugger-service;1"].getService(Ci.jsdIDebuggerService);
const io = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
const preferences = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
const json = Cc["@mozilla.org/dom/json;1"].getService(Ci.nsIJSON);
const directory = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);
const prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(Ci.nsIPromptService);

var debugOn = false;
var debugFlagStore, pauseChecked=true, src, store;
var trace = {__proto__: null};

function start()
{
    // for the traceFrame
  
    let request = new XMLHttpRequest();  
    request.open("GET", "chrome://domtracer/content/default.xhtml", false);  
    request.send(null);   
    
    var traceframe = document.getElementById("traceFrame");
    let address = "data:text/xml," + encodeURIComponent(request.responseText);
    
    traceframe.docShell.allowAuth = false;
    traceframe.docShell.allowImages = false;
    traceframe.docShell.allowJavascript = false;
    traceframe.docShell.allowMetaRedirects = false;
    traceframe.docShell.allowPlugins = false;
    traceframe.docShell.allowSubframes = false;
    traceframe.webNavigation.loadURI(address, 0, null, null, null);
    
    //for the varFrame
    //request = new XMLHttpRequest();  
    //request.open("GET", "chrome://bluedebug/content/varDefault.xhtml", false);  
    //request.send(null);
    //
    //var varframe = document.getElementById("varFrame");
    //address = "data:text/xml," + encodeURIComponent(request.responseText);
    //
    //varframe.docShell.allowAuth = false;
    //varframe.docShell.allowImages = false;
    //varframe.docShell.allowJavascript = false;
    //varframe.docShell.allowMetaRedirects = false;
    //varframe.docShell.allowPlugins = false;
    //varframe.docShell.allowSubframes = false;
    //varframe.webNavigation.loadURI(address, 0, null, null, null);
    
    debugOn = debug.isOn;
	if (!debugOn)
	{
		if ("asyncOn" in debug)
		{
			debug.asyncOn({onDebuggerActivated: onDebuggerActivated});
		}
		else
		{
			debug.on();
			onDebuggerActivated();
		}
	}
	else
		onDebuggerActivated();
}

function onDebuggerActivated()
{
	debug.scriptHook = scriptHook;
	debug.functionHook = scriptHook;
	debug.topLevelHook = scriptHook;

	debugFlagStore = debug.flags;
	debug.flags = ("DISABLE_OBJECT_TRACE" in Ci.jsdIDebuggerService ? Ci.jsdIDebuggerService.DISABLE_OBJECT_TRACE : 0);
}

function actOn(script,action)
{
    if (pauseChecked)
        return;
    
    if (action == "ret" && !(script.tag in trace))
        return;
    
    src = null;
    if (action == "exec" && !(script.tag in trace))
    {
        src = script.functionSource;
	//if (script.fileName.search(/chrome/i) == 0 || script.fileName.search(/file/i) == 0)
	//    return;
	
	if (script.fileName.search(/http/i)!=0)
	    return;
	
	/* If you want to proceed to the next code snippet only
	after seeing each executed script manually uncomment the command below*/
        
	//prompts.alert(window,script.fileName,src);
        trace[script.tag] = null;
	
	displayText(script, src);
    }
}

function makeRegexFromFile(fileName)
{
    // Regex File reading
    try
    {
	netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
    }
    catch (e)
    {
	alert("Permission to read file was denied.");
    }
    
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(fileName);
    if ( file.exists() == false )
    {
	alert("File does not exist");
    }
    
    var is = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    is.init(file,0x01, 00004, null);
    
    var sis = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
    sis.init(is);
    
    var output = sis.read(sis.available());
    //escapedOutput = escape(output);
    //regexString = escapedOutput.replace(/%0D%0A/g,"|");
    regexString = output.replace(/\r\n/g,"|");
    
    return regexString;
}


function displayText(script, source)
{
    let doc=document.getElementById("traceFrame").contentWindow.document;
    var lines=new Array();
    var temp;
                    
    lines=source.split("\n");   
    
    let mainArea = doc.getElementById("mainArea");
    let newNode = mainArea.cloneNode(true);
    newNode.removeAttribute("id");
    newNode.getElementsByClassName("scriptLineNum")[0].textContent = "Line: "+script.baseLineNumber;
    //newNode.getElementsByClassName("scriptCode")[0].textContent = source;  //remove this line when all code display is checked

    let scriptURLNode = newNode.getElementsByClassName("scriptURL")[0];
    scriptURLNode.href = scriptURLNode.textContent = "Path: "+script.fileName;
    scriptURLNode.lineNum = script.baseLineNumber;
    dispLine = script.baseLineNumber;   
  
    regexFilePath = document.getElementById("regexFileLabel").value;
    
    for (var i=0;i<lines.length;i++)
    {
	if (regexFilePath=="")
	{
	    newNode.getElementsByClassName("scriptCode")[0].textContent += "\n"+dispLine+"         "+lines[i];
	}
	else
	{
	    regexStr = makeRegexFromFile(regexFilePath);
	    temp=findText(lines[i], regexStr);
	    if (temp==false)
	    {
		//newNode.getElementsByClassName("scriptCode")[0].style.backgroundColour = "white";
	        newNode.getElementsByClassName("scriptCode")[0].textContent += "\n"+dispLine+"         "+lines[i];
	    }
	    else
	    {
		//newNode.getElementsByClassName("scriptCode")[0].style.backgroundColor = "yellow";
		newNode.getElementsByClassName("scriptCode")[0].textContent += "\n"+dispLine+"   >>>   "+lines[i];
	    }
	}
	dispLine++;
    }
    
    mainArea.parentNode.appendChild(newNode);
    return newNode;
}
  
function findText(line, text)
{
    var re= new RegExp(text);
    var m=re.exec(line);
    
    if(m==null)
    {
	return false;
    }
    else
    {
	return line.bold().fontcolor("Black");
    }
}

var scriptHook =
{
        onScriptCreated: function(script)
	{
	},
	onScriptDestroyed: function(script)
	{
	},
	onCall: function(frame, type)
	{
		if (type == Ci.jsdICallHook.TYPE_TOPLEVEL_START || type == Ci.jsdICallHook.TYPE_FUNCTION_CALL)
			actOn(frame.script,"exec");								//callhook>oncall>frame>script>tag
		else if (type == Ci.jsdICallHook.TYPE_TOPLEVEL_END || type == Ci.jsdICallHook.TYPE_FUNCTION_RETURN)
			actOn(frame.script,"ret");
	},
	prev: null,
	QueryInterface: XPCOMUtils.generateQI([Ci.jsdIScriptHook, Ci.jsdICallHook])
}

function clearTrace()
{
    let doc = document.getElementById("traceFrame").contentDocument;
    let toDelete = doc.createRange();
    let deleteHere = doc.getElementById("deleteHere");
    toDelete.setStartAfter(deleteHere);
    toDelete.setEndAfter(deleteHere.parentNode.lastChild);
    toDelete.deleteContents();
    
    trace = {_proto_ : null};
}

function stop()
{
    debug.scriptHook = null;
    debug.functionHook = null;
    debug.topLevelHook = null;
    debug.interruptHook = null;
    debug.flags = debugFlagStore;
    if (!debugOn)
	debug.off();
    alert("stopping");
}

function esc(v)
{
    v=v.replace(/&/g,"&amp;");
    v=v.replace(/>/g,"&gt;");
    v=v.replace(/</g,"&lt;");
    return v;
}

function getVar()
{
    var wnd = document.getElementById("varFrame").contentWindow;
    var doc = wnd.document;
    var temp=0,i,num,om1,om2;
    
    var tableArea = doc.getElementById("tableArea");
    
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
    if (wm==null || wm==undefined)
	alert("oops!");
    
    benumerator = wm.getEnumerator('navigator:browser');
    //var om = benumerator.getNext();
    //var om1 = om.getBrowser();		//browserwin
    //var om2 = om1.getBrowserAtIndex(0);	//tabbrowser
    if (!benumerator.hasMoreElements())
	alert("None!");
    while(benumerator.hasMoreElements())  
    {  
	alert("hi");
	om1 = benumerator.getNext(),  
	om2 = om1.getBrowser(),
	alert(om2.content.document.title);
	tabs = [];  
 
	for(var i=0; i<om2.browsers.length; i++)  
	{  
	    var browser = om2.getBrowserAtIndex(i);
	    for (x in browser)
	    {
		alert(x);
		tableArea.innerHTML+=x+="<br/>";
	    }
	    tableArea.innerHTML+="<br/>";
	    //tabs[i] = {  
	    //  'uri' : browser.currentURI.spec,  
	    //  'selected' : (tabbrowser.selectedTab == tabbrowser.mTabs[i])  
	}
   }  
 
    var windowObject = window.content;
    
    alert(browserWindow.document.title);
    
//    for (i in window.opener)
//    {
//	    tableArea.innerHTML+=i+"<br/>";
//    }
    
    alert("hi here!");
}

function toggle(button)
{
    pauseChecked=!pauseChecked;
}

function regex()
{
    let regexText = document.getElementById("regexTextbox").value;
    let doc=document.getElementById("traceFrame").contentWindow.document;
    
    let regexFinal = new RegExp(regexText, "g");
    let regexNew = regexText.bold();
    store=store.replace(regexFinal,regexFinal.exec(store)+"..");
    
    alert(store);
    
    alert(doc.getElementById("mainArea").innerHTML);
}